@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2012 Conrad Sanderson
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup fn_conv_to
@@ -44,12 +46,11 @@ conv_to<out_eT>::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_e
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const Proxy<T1> P(in.get_ref());
   
-  arma_debug_check( (X.n_elem != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
   
-  return out_eT(X.mem[0]);
+  return out_eT(Proxy<T1>::prefer_at_accessor ? P.at(0,0) : P[0]);
   }
 
 
@@ -65,14 +66,13 @@ conv_to<out_eT>::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const Proxy<T1> P(in.get_ref());
   
-  arma_debug_check( (X.n_elem != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
   
   out_eT out;
   
-  arrayops::convert_cx_scalar(out, X.mem[0]);
+  arrayops::convert_cx_scalar(out, (Proxy<T1>::prefer_at_accessor ? P.at(0,0) : P[0]));
   
   return out;
   }
@@ -90,12 +90,11 @@ conv_to<out_eT>::from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
   
-  const unwrap_cube<T1>  tmp(in.get_ref());
-  const Cube<in_eT>& X = tmp.M;
+  const ProxyCube<T1> P(in.get_ref());
   
-  arma_debug_check( (X.n_elem != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
   
-  return out_eT(X.mem[0]);
+  return out_eT(ProxyCube<T1>::prefer_at_accessor ? P.at(0,0,0) : P[0]);
   }
 
 
@@ -111,14 +110,13 @@ conv_to<out_eT>::from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
   
-  const unwrap_cube<T1>  tmp(in.get_ref());
-  const Cube<in_eT>& X = tmp.M;
+  const ProxyCube<T1> P(in.get_ref());
   
-  arma_debug_check( (X.n_elem != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
   
   out_eT out;
   
-  arrayops::convert_cx_scalar(out, X.mem[0]);
+  arrayops::convert_cx_scalar(out, (ProxyCube<T1>::prefer_at_accessor ? P.at(0,0,0) : P[0]));
   
   return out;
   }
@@ -137,6 +135,9 @@ class conv_to< Mat<out_eT> >
   template<typename in_eT, typename T1>
   inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
   
+  template<typename T1>
+  inline static Mat<out_eT> from(const SpBase<out_eT, T1>& in);
+  
   
   
   template<typename in_eT>
@@ -157,12 +158,12 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   Mat<out_eT> out(X.n_rows, X.n_cols);
   
-  arrayops::convert( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -178,18 +179,31 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   Mat<out_eT> out(X.n_rows, X.n_cols);
   
-  arrayops::convert_cx( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
 
 
 
+template<typename out_eT>
+template<typename T1>
+inline
+Mat<out_eT>
+conv_to< Mat<out_eT> >::from(const SpBase<out_eT, T1>& in)
+  {
+  arma_extra_debug_sigprint();
+  
+  return Mat<out_eT>(in.get_ref());
+  }
+
+
+
 template<typename out_eT>
 template<typename in_eT>
 inline
@@ -205,13 +219,7 @@ conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   if(N > 0)
     {
-          out_eT* out_mem = out.memptr();
-    const  in_eT*  in_mem = &(in[0]);
-    
-    for(uword i=0; i<N; ++i)
-      {
-      out_mem[i] = out_eT( in_mem[i] );
-      }
+    arrayops::convert( out.memptr(), &(in[0]), N );
     }
   
   return out;
@@ -228,23 +236,13 @@ conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  Mat<out_eT> out(in.size(), 1);
+  const uword N = uword( in.size() );
   
-  typename std::vector<in_eT>::const_iterator in_begin = in.begin();
-  typename std::vector<in_eT>::const_iterator in_end   = in.end();
+  Mat<out_eT> out(N, 1);
   
-  typename Mat<out_eT>::iterator out_begin = out.begin();
-  typename Mat<out_eT>::iterator out_end   = out.end();
-  
-  typename std::vector<in_eT>::const_iterator in_it;
-  typename Mat<out_eT>::iterator              out_it;
-  
-  for(in_it = in_begin, out_it = out_begin; (in_it != in_end) && (out_it != out_end); ++in_it, ++out_it)
+  if(N > 0)
     {
-          out_eT& out_elem = (*out_it);
-    const in_eT&  in_elem  = (*in_it);
-    
-    arrayops::convert_cx_scalar(out_elem, in_elem);
+    arrayops::convert_cx( out.memptr(), &(in[0]), N );
     }
   
   return out;
@@ -284,14 +282,14 @@ conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
   Row<out_eT> out(X.n_elem);
   
-  arrayops::convert( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -307,14 +305,14 @@ conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
   Row<out_eT> out(X.n_rows, X.n_cols);
   
-  arrayops::convert_cx( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -336,13 +334,7 @@ conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   if(N > 0)
     {
-          out_eT* out_mem = out.memptr();
-    const  in_eT*  in_mem = &(in[0]);
-    
-    for(uword i=0; i<N; ++i)
-      {
-      out_mem[i] = out_eT( in_mem[i] );
-      }
+    arrayops::convert( out.memptr(), &(in[0]), N );
     }
   
   return out;
@@ -359,23 +351,13 @@ conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  Row<out_eT> out( in.size() );
+  const uword N = uword( in.size() );
   
-  typename std::vector<in_eT>::const_iterator in_begin = in.begin();
-  typename std::vector<in_eT>::const_iterator in_end   = in.end();
+  Row<out_eT> out(N);
   
-  typename Row<out_eT>::iterator out_begin = out.begin();
-  typename Row<out_eT>::iterator out_end   = out.end();
-  
-  typename std::vector<in_eT>::const_iterator in_it;
-  typename Row<out_eT>::iterator              out_it;
-  
-  for(in_it = in_begin, out_it = out_begin; (in_it != in_end) && (out_it != out_end); ++in_it, ++out_it)
+  if(N > 0)
     {
-          out_eT& out_elem = (*out_it);
-    const in_eT&  in_elem  = (*in_it);
-    
-    arrayops::convert_cx_scalar(out_elem, in_elem);
+    arrayops::convert_cx( out.memptr(), &(in[0]), N );
     }
   
   return out;
@@ -415,14 +397,14 @@ conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
   Col<out_eT> out(X.n_elem);
   
-  arrayops::convert( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -438,14 +420,14 @@ conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
   Col<out_eT> out(X.n_rows, X.n_cols);
   
-  arrayops::convert_cx( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -467,13 +449,7 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   if(N > 0)
     {
-          out_eT* out_mem = out.memptr();
-    const  in_eT*  in_mem = &(in[0]);
-    
-    for(uword i=0; i<N; ++i)
-      {
-      out_mem[i] = out_eT( in_mem[i] );
-      }
+    arrayops::convert( out.memptr(), &(in[0]), N );
     }
   
   return out;
@@ -490,23 +466,13 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  Col<out_eT> out( in.size() );
+  const uword N = uword( in.size() );
   
-  typename std::vector<in_eT>::const_iterator in_begin = in.begin();
-  typename std::vector<in_eT>::const_iterator in_end   = in.end();
+  Col<out_eT> out(N);
   
-  typename Col<out_eT>::iterator out_begin = out.begin();
-  typename Col<out_eT>::iterator out_end   = out.end();
-  
-  typename std::vector<in_eT>::const_iterator in_it;
-  typename Col<out_eT>::iterator              out_it;
-  
-  for(in_it = in_begin, out_it = out_begin; (in_it != in_end) && (out_it != out_end); ++in_it, ++out_it)
+  if(N > 0)
     {
-          out_eT& out_elem = (*out_it);
-    const in_eT&  in_elem  = (*in_it);
-    
-    arrayops::convert_cx_scalar(out_elem, in_elem);
+    arrayops::convert_cx( out.memptr(), &(in[0]), N );
     }
   
   return out;
@@ -514,6 +480,30 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
 
 
 
+template<typename out_eT>
+class conv_to< SpMat<out_eT> >
+  {
+  public:
+  
+  template<typename T1>
+  inline static SpMat<out_eT> from(const Base<out_eT, T1>& in);
+  };
+
+
+
+template<typename out_eT>
+template<typename T1>
+inline
+SpMat<out_eT>
+conv_to< SpMat<out_eT> >::from(const Base<out_eT, T1>& in)
+  {
+  arma_extra_debug_sigprint();
+  
+  return SpMat<out_eT>(in.get_ref());
+  }
+
+
+
 //! conversion to Armadillo cubes from Armadillo BaseCube objects
 template<typename out_eT>
 class conv_to< Cube<out_eT> >
@@ -543,7 +533,7 @@ conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma
   
   Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices);
   
-  arrayops::convert( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -564,7 +554,7 @@ conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma
   
   Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices);
   
-  arrayops::convert_cx( out.memptr(), X.memptr(), out.n_elem );
+  arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
   return out;
   }
@@ -595,8 +585,8 @@ conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename a
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
@@ -606,13 +596,7 @@ conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename a
   
   if(N > 0)
     {
-          out_eT* out_mem = &(out[0]);
-    const  in_eT*   X_mem = X.memptr();
-    
-    for(uword i=0; i<N; ++i)
-      {
-      out_mem[i] = out_eT( X_mem[i] );
-      }
+    arrayops::convert( &(out[0]), X.memptr(), N );
     }
   
   return out;
@@ -629,28 +613,18 @@ conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename a
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  const unwrap<T1>      tmp(in.get_ref());
-  const Mat<in_eT>& X = tmp.M;
+  const quasi_unwrap<T1> tmp(in.get_ref());
+  const Mat<in_eT>& X  = tmp.M;
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
-  std::vector<out_eT> out(X.n_elem);
+  const uword N = X.n_elem;
   
-  typename Mat<in_eT>::const_iterator X_begin = X.begin();
-  typename Mat<in_eT>::const_iterator X_end   = X.end();
-
-  typename std::vector<out_eT>::iterator out_begin = out.begin();
-  typename std::vector<out_eT>::iterator out_end   = out.end();
+  std::vector<out_eT> out(N);
   
-  typename Mat<in_eT>::const_iterator    X_it;
-  typename std::vector<out_eT>::iterator out_it;
-  
-  for(X_it = X_begin, out_it = out_begin; (X_it != X_end) && (out_it != out_end); ++X_it, ++out_it)
+  if(N > 0)
     {
-          out_eT& out_elem = (*out_it);
-    const in_eT&  X_elem   = (*X_it);
-    
-    arrayops::convert_cx_scalar(out_elem, X_elem);
+    arrayops::convert_cx( &(out[0]), X.memptr(), N );
     }
   
   return out;
